unit LeastSqr;

{-------------------------------------------------------------------}
{-                                                                 -}
{-     Turbo Pascal Numerical Methods Toolbox                      -}
{-     Copyright (c) 1986, 87 by Borland International, Inc.       -}
{-                                                                 -}
{-  This unit provides procedures for modelling data
    with a function of a given type, given a set of data points.   -}
{-  changed Dec02 by ilh                                           -}
{-------------------------------------------------------------------}


interface

uses  Sysutils, Dialogs,
      DRegForm, DGlobalsUnit;
const
  TNNearlyZero = 1E-015;
  TNRowSize = 6;                { Maximum number of terms  }
                                { in Least Squares fit     }
  TNColumnSize = 50;            { Maximum number of data points  }

type
  TNColumnVector = array[1..TNColumnSize] of Double;
  TNRowVector = array[1..TNRowSize] of Double;
  TNmatrix = array[1..TNColumnSize] of TNRowVector;
  TNSquareMatrix =array[1..TNRowSize] of TNRowVector;
  TNString40 = string[40];
  FitType = (Expo, Fourier, Log, Poly, Power, User);

var
  XDataV2, YDataV2,
  XData, YData : TNColumnVector;    { Data points (X,Y)  }
  NumPoints : integer;              { # of points  }
{  NumTerms : integer;}               { # of terms in least squares  }
  ReallyXSolution, ReallyYSolution,
  Solution : TNRowVector;           { Coefficients of the l.s. fit  }
  ReallyXFit, ReallyYFit,
  YFit : TNColumnVector;            { Least squares solution  }
                                    { at XData points  }
  ReallyXResidual, ReallyYResidual,
  Residual : TNColumnVector;        { YFit - YData  }
  ReallyXStandardDeviationV2, ReallyYStandardDeviationV2:TNColumnVector;

  ReallyXStandardDeviation, ReallyYStandardDeviation,
  StandardDeviation : Double;        { Square root of variance;   }
  ReallyXVarianceV2, ReallyYVarianceV2:TNColumnVector;
  ReallyXVariance, ReallyYVariance,
  Variance : Double;                 { Indicates goodness of fit  }
  Error : byte;                     { Flags if something went wrong  }
  Fit : FitType;               { Indicates the type of fit requested }

  TestRatioArray:Array[1..8,1..20]of Double;

Procedure SetEmUpBoyo;
Function PerformALLLeastSquares(var TotalErrorValue: Double):Boolean;
                              {My added Totaler variable}
Function PerformLeastSquares(var TotalErrorValue: Double):Boolean;

{----------------------------------------------------------}
{- Output: XData, YData, Solution, YFit,                  -}
{-         Residual, Error                                -}
{-                                                        -}
{- This procedure initializes the variables to zero -}
{----------------------------------------------------------}
procedure Initialize(var XData    : TNColumnVector;
                     var YData    : TNColumnVector;
                     var YFit     : TNColumnVector;
                     var Residual : TNColumnVector;
                     var Solution : TNRowVector;
                     var Error    : byte);


{--------------------------------------------------------------}
{- Output: NumPoints, NumTerms, XData, YData                  -}
{-                                                            -}
{- This procedure reads in data from either the keyboard      -}
{- or a data file.  The number of data points (NumPoints),    -}
{- the data points (XData, YData), the number of terms        -}
{- (NumTerms) in the least squares fit are all read in here.  -}
{--------------------------------------------------------------}
procedure GetData(var NumPoints : integer;
                  var NumTerms  : integer;
                  var XData     : TNColumnVector;
                  var YData     : TNColumnVector);


{------------------------------------------------------------}
{- This procedure outputs the results to the device OutFile -}
{------------------------------------------------------------}
procedure Results(NumPoints         : integer;
              var XData             : TNColumnVector;
              var YData             : TNColumnVector;
                  NumTerms          : integer;
              var Solution          : TNRowVector;
              var YFit              : TNColumnVector;
              var Residual          : TNColumnVector;
                  StandardDeviation : Double;
                  Error             : byte;
                  Switch   : integer);


{Changes Fit into String name}
function ModuleName(Fit : FitType) : TNString40;

{-------------------------------------------------------------------}
{-   Input: NumPoints, XData, YData, NumTerms                      -}
{-   Output: Solution, YFit, Residuals, StandardDeviation, Error   -}
{-   Purpose:                                                      -}
{-    Given NumPoints data points of the form (X, Y), this         -}
{-    procedure finds the least square solution of NumTerms terms  -}
{-    (NumTerms <= NumPoints) to the matrix equation AC = B       -}
{-    where A is a NumPoints by NumTerms matrix, B is a            -}
{-    NumPoints vector and C is the least squares solution.  The   -}
{-    elements of A are A[i, j] = Tj(X[i]) where Tj is the jth     -}
{-    basis vector and X[i] is the X-value of the ith data point.  -}
{-    The basis vectors are created by a separate include file,    -}
{-    or module.  The choice of module will determine whether      -}
{-    the least squares solution is a polynomial fit, trigono-     -}
{-    metric fit (Fourier series), power fit (e.g. Y=ax^b, where   -}
{-    b is fractional), exponential fit (e.g. Y=a-Exp(bx)), or     -}
{-    logarithmic fit (e.g. Y=a-Ln(bx)).  The user may also        -}
{-    create modules for other functional forms.  See the          -}
{-    documentation for details.                                   -}
{-                                                                 -}
{- User-Defined Types:
   TNColumnVector = array[1..TNColumnSize] of real;     -}
{- TNRowVector = array[1..TNRowSize] of real;           -}
{-                                                                 -}
{- Global Variables:
   NumPoints         : integer; Number of data points -}
{- XData             : TNColumnVector; X-value data   -}
{- YData             : TNColumnVector; Y-value data   -}
{- NumTerms          : integer; Number of terms in    -}
{_                              least squares fit     -}
{- Solution          : TNRowVector; Least squares     -}
{_                                 solution in the   -}
{-                                given basis       -}
{- YFit              : TNColumnVector; Y-values       -}
{-                                   predicted by   -}
{-                                  the LS solution -}
{- Residuals         :TNColumnVector Difference       -}
{-                               between predicted   -}
{-                               and actual Y values -}
{- StandardDeviation : real; Root of variance         -}
{- Error             : byte; Indicates an error       -}
{-                                                            -}
{- Errors: 0: No errors                                       -}
{-         1: NumPoints < 2                                   -}
{-        2: NumTerms < 1                                    -}
{-        3: NumTerms > NumPoints                            -}
{-       4: solution not possible (exact reason             -}
{-          will depend upon the particular basis)          -}
{-                                                          -}
{------------------------------------------------------------}
procedure LeastSquares(NumPoints         : integer;
                   var XData             : TNColumnVector;
                   var YData             : TNColumnVector;
                       NumTerms          : integer;
                   var Solution          : TNRowVector;
                   var YFit              : TNColumnVector;
                   var Residuals         : TNColumnVector;
                   var StandardDeviation : Double;
                   var Variance          : Double;
                   var Error             : byte;
                       Fit               : FitType);

implementation

{uses DRegForm, DGlobalsUnit;}

Procedure SetEmUpBoyo;
var
{J,  I:Integer;}
{XOrigin,YOrigin, }
  DXFXScale,DXFYScale,
  dYScaleRatio,dXScaleRatio,
  X1Reg,X2Reg,Y1Reg,Y2Reg:Double;
Begin   {XOrigin=1  dXScaleRatio=2 YOrigin=11  dYScaleRatio=12}
If (LSRegPntArray[1].LongXx <  LSRegPntArray[4].LongXx)
 then begin
  LowerDigiLocoX:=LSRegPntArray[1].X;
  XOrigin:=LSRegPntArray[1].LongXx;{LongXx}{ ((aRegPnt4.LocoX))}
 end else begin
   LowerDigiLocoX:=LSRegPntArray[4].X;
   XOrigin:=LSRegPntArray[4].LongXx;{LongXx}{ ((aRegPnt4.LocoX))}
 end;
If (LSRegPntArray[3].LatYy <  LSRegPntArray[4].LatYy)
 then begin
   LowerDigiLocoX:=LSRegPntArray[3].Y;
   YOrigin:=LSRegPntArray[3].LatYy;{LatYy} {((aRegPnt4.LocoY))}
 end else begin
   LowerDigiLocoY:=LSRegPntArray[4].Y;
   YOrigin:=LSRegPntArray[4].LatYy;{LatYy} {((aRegPnt4.LocoY))}
 end;

 X1Reg:= abs(LSRegPntArray[1].LongXx- LSRegPntArray[2].LongXx);
 X2Reg:= abs(LSRegPntArray[1].X - LSRegPntArray[2].X);
 Y1Reg:= abs(LSRegPntArray[1].LatYy- LSRegPntArray[4].LatYy);
 Y2Reg:= abs(LSRegPntArray[1].Y - LSRegPntArray[4].Y);
 DXFXScale:=(X2Reg /  X1Reg); {World / Digitizer}
 DXFYScale:=(Y2Reg / Y1Reg);
       dXScaleRatio:= (DXFXScale );
       dYScaleRatio:= (DXFYScale );
TestRatioArray[1,2]:= dXScaleRatio;
TestRatioArray[1,12]:= dYScaleRatio;
LSRegPntArray[1].DigiX :=XOrigin+((LSRegPntArray[1].X-LowerDigiLocoX) * dXScaleRatio);
LSRegPntArray[1].DigiY :=YOrigin+((LSRegPntArray[1].Y-LowerDigiLocoY) * dYScaleRatio);



 X1Reg:= abs(LSRegPntArray[1].LongXx- LSRegPntArray[2].LongXx);
 X2Reg:= abs(LSRegPntArray[1].X - LSRegPntArray[2].X);
 Y1Reg:= abs(LSRegPntArray[2].LatYy- LSRegPntArray[3].LatYy);
 Y2Reg:= abs(LSRegPntArray[2].Y - LSRegPntArray[3].Y);
 DXFXScale:=(X2Reg /  X1Reg); {World / Digitizer}
 DXFYScale:=(Y2Reg / Y1Reg);
       dXScaleRatio:= (DXFXScale );
       dYScaleRatio:= (DXFYScale );
TestRatioArray[2,2]:= dXScaleRatio;
TestRatioArray[2,12]:= dYScaleRatio;
LSRegPntArray[2].DigiX :=XOrigin+((LSRegPntArray[2].X-LowerDigiLocoX) * dXScaleRatio);
LSRegPntArray[2].DigiY :=YOrigin+((LSRegPntArray[2].Y-LowerDigiLocoY) * dYScaleRatio);



 X1Reg:= abs(LSRegPntArray[4].LongXx- LSRegPntArray[3].LongXx);
 X2Reg:= abs(LSRegPntArray[4].X - LSRegPntArray[3].X);
 Y1Reg:= abs(LSRegPntArray[2].LatYy- LSRegPntArray[3].LatYy);
 Y2Reg:= abs(LSRegPntArray[2].Y - LSRegPntArray[3].Y);
 DXFXScale:=(X2Reg /  X1Reg); {World / Digitizer}
 DXFYScale:=(Y2Reg / Y1Reg);
       dXScaleRatio:= (DXFXScale );
       dYScaleRatio:= (DXFYScale );
TestRatioArray[3,2]:= dXScaleRatio;
TestRatioArray[3,12]:= dYScaleRatio;
LSRegPntArray[3].DigiX :=XOrigin+((LSRegPntArray[3].X-LowerDigiLocoX) * dXScaleRatio);
LSRegPntArray[3].DigiY :=YOrigin+((LSRegPntArray[3].Y-LowerDigiLocoY) * dYScaleRatio);



 X1Reg:= abs(LSRegPntArray[4].LongXx- LSRegPntArray[3].LongXx);
 X2Reg:= abs(LSRegPntArray[4].X - LSRegPntArray[3].X);
 Y1Reg:= abs(LSRegPntArray[1].LatYy- LSRegPntArray[4].LatYy);
 Y2Reg:= abs(LSRegPntArray[1].Y - LSRegPntArray[4].Y);
 DXFXScale:=(X2Reg /  X1Reg); {World / Digitizer}
 DXFYScale:=(Y2Reg / Y1Reg);
       dXScaleRatio:= (DXFXScale );
       dYScaleRatio:= (DXFYScale );
TestRatioArray[4,2]:= dXScaleRatio;
TestRatioArray[4,12]:= dYScaleRatio;
LSRegPntArray[4].DigiX :=XOrigin+((LSRegPntArray[4].X-LowerDigiLocoX) * dXScaleRatio);
LSRegPntArray[4].DigiY :=YOrigin+((LSRegPntArray[4].Y-LowerDigiLocoY) * dYScaleRatio);


If (NumPoints=8) then {do them too}
begin
 X1Reg:= abs(LSRegPntArray[1].LongXx- LSRegPntArray[5].LongXx);
 X2Reg:= abs(LSRegPntArray[1].X - LSRegPntArray[5].X);
 Y1Reg:= abs(LSRegPntArray[5].LatYy- LSRegPntArray[7].LatYy);
 Y2Reg:= abs(LSRegPntArray[5].Y - LSRegPntArray[7].Y);
 DXFXScale:=(X2Reg /  X1Reg); {World / Digitizer}
 DXFYScale:=(Y2Reg / Y1Reg);
       dXScaleRatio:= (DXFXScale );
       dYScaleRatio:= (DXFYScale );
TestRatioArray[5,2]:= dXScaleRatio;
TestRatioArray[5,12]:= dYScaleRatio;
LSRegPntArray[5].DigiX :=XOrigin+((LSRegPntArray[5].X-LowerDigiLocoX) * dXScaleRatio);
LSRegPntArray[5].DigiY :=YOrigin+((LSRegPntArray[5].Y-LowerDigiLocoY) * dYScaleRatio);



 X1Reg:= abs(LSRegPntArray[6].LongXx- LSRegPntArray[8].LongXx);
 X2Reg:= abs(LSRegPntArray[6].X - LSRegPntArray[8].X);
 Y1Reg:= abs(LSRegPntArray[1].LatYy- LSRegPntArray[6].LatYy);
 Y2Reg:= abs(LSRegPntArray[1].Y - LSRegPntArray[6].Y);
 DXFXScale:=(X2Reg /  X1Reg); {World / Digitizer}
 DXFYScale:=(Y2Reg / Y1Reg);
       dXScaleRatio:= (DXFXScale );
       dYScaleRatio:= (DXFYScale );
TestRatioArray[6,2]:= dXScaleRatio;
TestRatioArray[6,12]:= dYScaleRatio;
LSRegPntArray[6].DigiX :=XOrigin+((LSRegPntArray[6].X-LowerDigiLocoX) * dXScaleRatio);
LSRegPntArray[6].DigiY :=YOrigin+((LSRegPntArray[6].Y-LowerDigiLocoY) * dYScaleRatio);


 X1Reg:= abs(LSRegPntArray[4].LongXx- LSRegPntArray[7].LongXx);
 X2Reg:= abs(LSRegPntArray[4].X - LSRegPntArray[7].X);
 Y1Reg:= abs(LSRegPntArray[5].LatYy- LSRegPntArray[7].LatYy);
 Y2Reg:= abs(LSRegPntArray[5].Y - LSRegPntArray[7].Y);
 DXFXScale:=(X2Reg /  X1Reg); {World / Digitizer}
 DXFYScale:=(Y2Reg / Y1Reg);
       dXScaleRatio:= (DXFXScale );
       dYScaleRatio:= (DXFYScale );
TestRatioArray[7,2]:= dXScaleRatio;
TestRatioArray[7,12]:= dYScaleRatio;
LSRegPntArray[7].DigiX :=XOrigin+((LSRegPntArray[7].X-LowerDigiLocoX) * dXScaleRatio);
LSRegPntArray[7].DigiY :=YOrigin+((LSRegPntArray[7].Y-LowerDigiLocoY) * dYScaleRatio);


 X1Reg:= abs(LSRegPntArray[6].LongXx- LSRegPntArray[8].LongXx);
 X2Reg:= abs(LSRegPntArray[6].X - LSRegPntArray[8].X);
 Y1Reg:= abs(LSRegPntArray[2].LatYy- LSRegPntArray[8].LatYy);
 Y2Reg:= abs(LSRegPntArray[2].Y - LSRegPntArray[8].Y);
 DXFXScale:=(X2Reg /  X1Reg); {World / Digitizer}
 DXFYScale:=(Y2Reg / Y1Reg);
       dXScaleRatio:= (DXFXScale );
       dYScaleRatio:= (DXFYScale );
TestRatioArray[8,2]:= dXScaleRatio;
TestRatioArray[8,12]:= dYScaleRatio;
LSRegPntArray[8].DigiX :=XOrigin+((LSRegPntArray[8].X-LowerDigiLocoX) * dXScaleRatio);
LSRegPntArray[8].DigiY :=YOrigin+((LSRegPntArray[8].Y-LowerDigiLocoY) * dYScaleRatio);
end;
(*
For J:= 1 to NumPoints do begin
For I:= 1 to NumPoints do
begin
TestRatioArray[J,(I+2)]:=
TestRatioArray[J,1]+((LSRegPntArray[I].X-LowerDigiLocoX) * TestRatioArray[J,2]{RX});
TestRatioArray[J,(I+12)]:=
TestRatioArray[J,11]+((LSRegPntArray[I].Y-LowerDigiLocoY) * TestRatioArray[J,12]{RY});
end;
LSRegPntArray[J].LocoX := TestRatioArray[J,(3)];
LSRegPntArray[J].LocoY := TestRatioArray[J,(13)];
end;*)
End;

Function PerformALLLeastSquares(var TotalErrorValue: Double):Boolean;
var I,J:Integer;
Begin
  {Do SetEmUpBoyo to get BASE RATIO for that set TestRatioArray[]}
  SetEmUpBoyo;
  Fit := Poly;
  NumTerms:=2;


  For I:= 1 to NumPoints do
  Begin   {XOrigin=1  dXScaleRatio=2 YOrigin=11  dYScaleRatio=12}
  XDataV2[I]:=TestRatioArray[I,I+2];
  YDataV2[I]:=TestRatioArray[I,I+12];
    For J:= 1 to NumPoints do
    Begin {Compute Coords for each point}
    If (I<>J) then begin
      Initialize(XData, YData, YFit, Residual, Solution, Error);
      XData[1]:=TestRatioArray[I,I+2];
      YData[1]:=TestRatioArray[I,I+12];
      XData[2]:=TestRatioArray[I,J+2];
      YData[2]:=TestRatioArray[I,J+12];
       {Do LS for EACH SET}
      LeastSquares(2{NumPoints}, XData, YData, NumTerms, Solution,
               YFit, Residual,
               StandardDeviation, Variance, Error, Fit);
      Results(2{NumPoints}, XData, YData, 2{NumTerms}, Solution, YFit,
          Residual, StandardDeviation, Error,0);
       {Total Errors for SET}
      ReallyXResidual[I]:= ReallyXResidual[I]+Residual[1]; {Residual TNColumnVector}
      ReallyXResidual[I]:=  ReallyXResidual[I]+Residual[2];
      ReallyXStandardDeviationV2[I]:= ReallyXStandardDeviationV2[I]+StandardDeviation; {Double}
      ReallyXVarianceV2[I]:=ReallyXVarianceV2[I]+Variance;
      ReallyXFit[I]:=ReallyXFit[I]+YFit[1]+YFit[2];
      {Must be done for X then for Y ...}
      Initialize(XData, YData, YFit, Residual, Solution, Error);
      YData[1]:=TestRatioArray[I,I+2];
      XData[1]:=TestRatioArray[I,I+12];
      YData[2]:=TestRatioArray[I,J+2];
      XData[2]:=TestRatioArray[I,J+12];
       {Do LS for EACH SET}
      LeastSquares(2{NumPoints}, XData, YData, NumTerms, Solution,
               YFit, Residual,
               StandardDeviation, Variance, Error, Fit);
      Results(2{NumPoints}, XData, YData, 2{NumTerms}, Solution, YFit,
          Residual, StandardDeviation, Error,1);
       {Total Errors for SET}
      ReallyYResidual[I]:= ReallyYResidual[I]+Residual[1]; {Residual TNColumnVector}
      ReallyYResidual[I]:=  ReallyYResidual[I]+Residual[2];
      ReallyYStandardDeviationV2[I]:= ReallyYStandardDeviationV2[I]+StandardDeviation; {Double}
      ReallyYVarianceV2[I]:=ReallyYVarianceV2[I]+Variance;
      ReallyYFit[I]:=ReallyYFit[I]+YFit[1]+YFit[2];
      end;{of begin I<>J}
    End;{of 1 set I for all J}
    ReallyXResidual[I]:=ReallyXResidual[I]/NumPoints;
    ReallyYResidual[I]:=ReallyYResidual[I]/NumPoints;
    ReallyYFit[I]:= ReallyYFit[I]/(NumPoints*2);
    ReallyXFit[I]:= ReallyXFit[I]/(NumPoints*2);
    ReallyXStandardDeviationV2[I]:=ReallyXStandardDeviationV2[I]/NumPoints;
    ReallyYStandardDeviationV2[I]:=ReallyYStandardDeviationV2[I]/NumPoints;
  End;{I}
  {Average The Ratios to Determine XRatio and Y Ratio}
       For I:= 1 to NumPoints do
     ReallyXStandardDeviation:=
     ReallyXStandardDeviation+ReallyXStandardDeviationV2[I];
       For I:= 1 to NumPoints do
     ReallyYStandardDeviation:=
     ReallyYStandardDeviation+ReallyYStandardDeviationV2[I];
    {TotalAllErrors     GetPercentOfErrorForEachSet}
      Results(NumPoints, XDataV2, YDataV2,
              2{NumTerms}, Solution, ReallyXFit,
          ReallyXResidual, ReallyXStandardDeviation,
           Error,0);
      Results(NumPoints, YDataV2, XDataV2,
              2{NumTerms}, Solution, ReallyYFit,
          ReallyYResidual, ReallyYStandardDeviation,
           Error,1);
  TotalErrorValue:=0;
  PerformALLLeastSquares:=True;
End;

{Must be done for X then for Y ...}
Function PerformLeastSquares(var TotalErrorValue: Double):Boolean;
var MName:String;
I:Integer;
begin { PerformLeastSquares}
    {initializes the variables to zero}
  Initialize(XData, YData, YFit, Residual, Solution, Error);
{  number of data points (NumPoints),
 the number of terms (NumTerms) in the least squares fit
  the data points (XData, YData),}
{  GetData(NumPoints, NumTerms, XData, YData);}

  {Do SetEmUpBoyo to get BASE RATIO for that set TestRatioArray[]}
  SetEmUpBoyo;
  For I:= 1 to NumPoints do
  Begin
    XData[I]:=LSRegPntArray[I].DigiX;
    YData[I]:=LSRegPntArray[I].DigiY;
  End;
  {- The value of Fit determines the type of fit requested -}
  Fit := Poly;
  MName:= ModuleName(Fit);

  LeastSquares(NumPoints, XData, YData, NumTerms, Solution,
               YFit, Residual,
               StandardDeviation, Variance, Error, Fit);
   {Need to rewrite to output to the GRID}
  Results(NumPoints, XData, YData, NumTerms, Solution, YFit,
          Residual, StandardDeviation, Error,0);
  {---------------------------------------------------------}
  {SAVE the data as it is OVERWRITTEN}
     for I := 1 to 8 do
     LSRegPntArray[I].SolutionX :=1;
     for I := 1 to NumTerms do
     LSRegPntArray[I].SolutionX := Solution[I];
   for I := 1 to NumTerms do
       ReallyXSolution[I]:= Solution[I]; {TNRowVector}
   for I := 1 to NumPoints do
   Begin
     ReallyXFit[I]:= YFit[I]; {YFit TNColumnVector}
     ReallyXResidual[I]:= Residual[I]; {Residual TNColumnVector}
   End;
   ReallyXStandardDeviation:= StandardDeviation; {Double}
   ReallyXVariance:=Variance; {Double}
  {---------------------------------------------------------}
{Must be done for X then for Y ...}
  Initialize(XData, YData, YFit, Residual, Solution, Error);
  For I:= 1 to NumPoints do
  Begin
    XData[I]:=LSRegPntArray[I].DigiY;
    YData[I]:=LSRegPntArray[I].DigiX;
  End;

  LeastSquares(NumPoints, XData, YData, NumTerms, Solution,
               YFit, Residual,
               StandardDeviation, Variance, Error, Fit);
   {Need to rewrite to output to the GRID}
  Results(NumPoints, XData, YData, NumTerms, Solution, YFit,
          Residual, StandardDeviation, Error,1);
  {---------------------------------------------------------}
  {SAVE the data as it is OVERWRITTEN}
     for I := 1 to 8 do
     LSRegPntArray[I].SolutionY :=1;
     for I := 1 to NumTerms do
     LSRegPntArray[I].SolutionY := Solution[I];
   for I := 1 to NumTerms do
       ReallyYSolution[I]:= Solution[I]; {TNRowVector}
   for I := 1 to NumPoints do
   Begin
     ReallyYFit[I]:= YFit[I]; {YFit TNColumnVector}
     ReallyYResidual[I]:= Residual[I]; {Residual TNColumnVector}
   End;
   ReallyYStandardDeviation:= StandardDeviation; {Double}
   ReallyYVariance:=Variance; {Double}
  {---------------------------------------------------------}
  TotalErrorValue:=0;
  PerformLeastSquares:=True;
end; {PerformLeastSquares }


{----------------------------------------------------------}
{- Output: XData, YData, Solution, YFit,                  -}
{-         Residual, Error                                -}
{-                                                        -}
{- This procedure initializes the above variables to zero -}
{----------------------------------------------------------}
procedure Initialize(var XData    : TNColumnVector;
                     var YData    : TNColumnVector;
                     var YFit     : TNColumnVector;
                     var Residual : TNColumnVector;
                     var Solution : TNRowVector;
                     var Error    : byte);
begin
  FillChar(XData, SizeOf(XData), 0);
  FillChar(YData, SizeOf(YData), 0);
  FillChar(Solution, SizeOf(Solution), 0);
  FillChar(YFit, SizeOf(YFit), 0);
  FillChar(Residual, SizeOf(Residual), 0);
  FillChar(ReallyXFit, SizeOf(ReallyXFit), 0);
  FillChar(ReallyYFit, SizeOf(ReallyYFit), 0);
  FillChar(XDataV2, SizeOf(XDataV2), 0);
  FillChar(YDataV2, SizeOf(YDataV2), 0);
  FillChar(ReallyXResidual, SizeOf(ReallyXResidual), 0);
  FillChar(ReallyXStandardDeviationV2, SizeOf(ReallyXStandardDeviationV2), 0);
  FillChar(ReallyXVarianceV2, SizeOf(ReallyXVarianceV2), 0);
  FillChar(ReallyYResidual, SizeOf(ReallyYResidual), 0);
  FillChar(ReallyYStandardDeviationV2, SizeOf(ReallyYStandardDeviationV2), 0);
  FillChar(ReallyYVarianceV2, SizeOf(ReallyYVarianceV2), 0);

  Error := 0;
end; { procedure Initialize }


{--------------------------------------------------------------}
{- Output: NumPoints, NumTerms, XData, YData                  -}
{-                                                            -}
{- This procedure reads in data from either the keyboard      -}
{- or a data file.  The number of data points (NumPoints),    -}
{- the data points (XData, YData), the number of terms        -}
{- (NumTerms) in the least squares fit are all read in here.  -}
{--------------------------------------------------------------}
procedure GetData(var NumPoints : integer;
                  var NumTerms  : integer;
                  var XData     : TNColumnVector;
                  var YData     : TNColumnVector);
var I:Integer;
Begin
{    Write('Number of terms in the least squares fit (<= ',
             NumPoints, ')? ');     Readln(NumTerms);}
  For I:= 1 to NumPoints do
  Begin
    XData[I]:=LSRegPntArray[I].LongXx;
    YData[I]:=LSRegPntArray[I].LatYy;
  End;
End;


{------------------------------------------------------------}
{- This procedure outputs the results to the device OutFile -}
{------------------------------------------------------------}
procedure Results(NumPoints         : integer;
              var XData             : TNColumnVector;
              var YData             : TNColumnVector;
                  NumTerms          : integer;
              var Solution          : TNRowVector;
              var YFit              : TNColumnVector;
              var Residual          : TNColumnVector;
                  StandardDeviation : Double;
                  Error             : byte;
                  Switch:Integer);
var
  Index : integer;
begin              { Display Open dialog box }
  RegistrationForm.LSHiddenMemo.Lines.Add(' ');
    RegistrationForm.LSHiddenMemo.Lines.Add('******************************************');
    RegistrationForm.LSHiddenMemo.Lines.Add('******************************************');
    RegistrationForm.LSHiddenMemo.Lines.Add('The Data Points:');
    If Switch=0 then {X,Y else Y,x}
    RegistrationForm.LSHiddenMemo.Lines.Add('     X                   Y')
    else     RegistrationForm.LSHiddenMemo.Lines.Add('     Y                   X');
    for Index := 1 to NumPoints do
      RegistrationForm.LSHiddenMemo.Lines.Add(
      Floattostrf(XData[Index],ffFixed,8,3)+'          '+
      Floattostrf( YData[Index],ffFixed,12,7));
    RegistrationForm.LSHiddenMemo.Lines.Add(' ');
    {  if Error >= 1 then DisplayError;}
    case Error of
      0 : begin
        RegistrationForm.LSHiddenMemo.Lines.Add('*----------------------------------------*');
        RegistrationForm.LSHiddenMemo.Lines.Add( ModuleName(Fit));
        RegistrationForm.LSHiddenMemo.Lines.Add('*----------------------------------------*');
        RegistrationForm.LSHiddenMemo.Lines.Add(' ');
       RegistrationForm.LSHiddenMemo.Lines.Add('Coefficients in least squares approximation:');
        for Index := 1 to NumTerms do
          RegistrationForm.LSHiddenMemo.Lines.Add(
          '   Coefficient '+ Inttostr(Index-1)+ ': '+
                  Floattostr(Solution[Index]));
        RegistrationForm.LSHiddenMemo.Lines.Add(' ');
        If Switch=0 then {X,Y else Y,x}
        RegistrationForm.LSHiddenMemo.Lines.Add('     X'+ '        '+
         'Least Squares Fit'+ '               '+'Residual')
         else         RegistrationForm.LSHiddenMemo.Lines.Add('     Y'+ '        '+
         'Least Squares Fit'+ '               '+'Residual');
        for Index := 1 to NumPoints do
          RegistrationForm.LSHiddenMemo.Lines.Add(
          Floattostrf(XData[Index],ffFixed,8,4)+ '    '+
          Floattostrf(YFit[Index],ffFixed,8,4)+'                    ' +
          Floattostrf(Residual[Index],ffFixed,18,14));
        RegistrationForm.LSHiddenMemo.Lines.Add(' ');
        RegistrationForm.LSHiddenMemo.Lines.Add('Standard Deviation : '+
                Floattostrf(StandardDeviation,ffFixed,18,14));
        RegistrationForm.LSHiddenMemo.Lines.Add('^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^');                
        RegistrationForm.LSHiddenMemo.Lines.Add(' ');
      end;

      1 : RegistrationForm.LSHiddenMemo.Lines.Add('There must be more than one data point.');

      2 : RegistrationForm.LSHiddenMemo.Lines.Add(
          'The number of terms in the fit must be greater than zero.');

      3 : begin
        RegistrationForm.LSHiddenMemo.Lines.Add('The number of terms in the solution must be');
        RegistrationForm.LSHiddenMemo.Lines.Add('less than the number of data points.');
      end;

      4 :RegistrationForm.LSHiddenMemo.Lines.Add(
          'There is no least squares solution to this set of data.');
    end; { case }
end; { procedure Results }

{$I Least1.inc}      { Include procedure code }
{$I Least2.inc}      { LeastSqr }
end.
