{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzGlobal;          {buzStpl  buzGlobal}
{Combines GCTPC, paksz,
 cproj (code), proj (list), report (error report)
 data requirements
 All Called Projections data requirements}

interface
uses SysUtils,Math;

var
buzMessageY,buzMessageX: Integer;
buzProjectPath, buzProjectName:String;
(*  from Proj*)
const
 buzHelpContextl: Integer = 123;
  GEO: Integer = 0; //0 = Geographic
  UTM: Integer = 1; //1 = Universal Transverse Mercator (UTM)
  SPCS: Integer = 2; //2 = State Plane Coordinates
  ALBERS: Integer = 3; //3 = Albers Conical Equal Area
  LAMCC: Integer = 4; //4 = Lambert Conformal Conic
  MERCAT: Integer = 5; //5 = Mercator
  PS: Integer = 6; //6 = Polar Stereographic
  POLYC: Integer = 7; //7 = Polyconic
  EQUIDC: Integer = 8; //8 = Equidistant Conic
  TM: Integer = 9; //9 = Transverse Mercator
  STEREO: Integer = 10; //10 = Stereographic
  LAMAZ: Integer = 11; //11 = Lambert Azimuthal Equal Area
  AZMEQD: Integer = 12; //12 = Azimuthal Equidistant
  GNOMON: Integer = 13; //13 = Gnomonic
  ORTHO: Integer = 14; //14 = Orthographic
  GVNSP: Integer = 15; //15 = General Vertical Near-Side Perspective
  SNSOID: Integer = 16; //16 = Sinusiodal
  EQRECT: Integer = 17; //17 = Equirectangular
  MILLER: Integer = 18; //18 = Miller Cylindrical
  VGRINT: Integer = 19; //19 = Van der Grinten
  HOM: Integer = 20; //20 = (Hotine) Oblique Mercator
  ROBIN: Integer = 21; //21 = Robinson
  SOM: Integer = 22; //22 = Space Oblique Mercator (SOM)
  ALASKA: Integer = 23; // 23 = Alaska Conformal
  GOOD: Integer = 24; //24 = Interrupted Goode Homolosine
  MOLL: Integer = 25; //25 = Mollweide
  IMOLL: Integer = 26; //26 = Interrupted Mollweide
  HAMMER: Integer = 27; //27 = Hammer
  WAGIV: Integer = 28; //28 = Wagner IV
  WAGVII: Integer = 29; //29 = Wagner VII
  OBEQA: Integer = 30; //30 = Oblated Equal Area
  USDEF: Integer = 99; //99 = User defined

  IN_BREAK: Integer = -2;
{  COEFCT: Integer = 15;} //  projection coefficient count
{  PROJCT: Integer = 30;} //  projection count  ?
{  DATMCT: Integer = 20; }// ???? datum count   ?
{  SPHDCT: Integer = 31;} //  spheroid count
{  MAXPROJ: Integer = 31;} //  Maximum projection number
  MAXUNIT: Integer = 5; //  Maximum unit code number
{  GEO_TERM: Integer = 0;} //  Array index for print-to-term flag
{  GEO_FILE: Integer = 1;} //  Array index for print-to-file flag
  GEO_TRUE: Integer = 1;
    //*  True value for geometric true/false flags */
  GEO_FALSE: Integer = -1;
    //*  False val for geometric true/false flags */
{iter: integer = 0;} //* First time flag    */
const
{#define}{already in System PI:Double=  3.141592653589793238;}
 HALF_PI: Double = (PI * 0.5);
 TWO_PI: Double = (PI * 2.0);
 EPSLN: Double = 1.0E-10;
 R2D: Double = 57.2957795131;
 D2R: Double = 0.01745329251994328;
 S2R: Double = 4.848136811095359E-6;
 OK: Integer = 0;
 ERROR: Integer = -1;


{from CPROJ}
 MAX_VAL: Integer = 4;
 MAXLONG: Integer = 2147483647;
 DBLLONG: Double = 4.61168601E18;

type      {0..14 + 8 for base info
            15:Projection
            16:Units
            17:UTM Zone
            18:Datum
            19:Spheroid
            20:State Plane Zone
            21: Z units
            22: OUTPUT Units added Dec02}
  parameters = array[0..22] of double;
{  parray31 = array[0..31] of Integer;
  ppa = array[0..31, 0..16] of double;}
var
{inpj,outpj: parray31;} //* input projection array
  //array [0..31{MAXPROJ + 1}] of Integer;
{indat,outdat: parray31;}//* input datum array
  //array [0..31] of Integer;
{inzn,outzn: parray31;} // zone array
  //array [0..31] of Integer;
{pdin,pdout: ppa;  }
  //array [0..31,0..16]{[MAXPROJ + 1][15]} of double;  //* input projection parm array  */
inparm,  outparm: parameters; //array [0..31]of double;
{parm:parameters;}{array [0..16]of double;}

iflg, inv_trans,for_trans:integer;{Passage rather than Function calls}
bIgnoreErrorsCB,bIgnoreDatumCB:Boolean;
// Table of unit codes as specified by state
// laws as of 2/1/92 for NAD 1983 State Plane projection,
//    1 = U.S. Survey Feet,
//    2 = Meters,
//    5 = International Feet

{static long}
  NADUT: array[1..134 {134}] of Integer =
   (1, 5, 1, 1, 5, 1, 1, 1, 1, 2,
    1, 1, 1, 2, 1, 1, 2, 2, 1, 1,
    5, 2, 1, 2, 5, 1, 2, 2, 2, 1,
    1, 1, 5, 2, 1, 5, 2, 2, 5, 2,
    1, 1, 5, 2, 2, 1, 2, 1, 2, 2, 1, 2, 2, 2,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0);
  NAD27: array[1..134 {134}] of Integer =(101, 102,
  5010, 5300,  201,  202,  203,  301,  302,  401,  402,  403,
   404,  405,  406,  407,  501,  502,  503,  600,  700,  901,
   902,  903, 1001, 1002, 5101, 5102, 5103, 5104, 5105, 1101,
  1102, 1103, 1201, 1202, 1301, 1302, 1401, 1402, 1501, 1502,
  1601, 1602, 1701, 1702, 1703, 1801, 1802, 1900, 2001, 2002,
  2101, 2102, 2103, 2111, 2112, 2113, 2201, 2202, 2203, 2301,
  2302, 2401, 2402, 2403, 2501, 2502, 2503, 2601, 2602, 2701,
  2702, 2703, 2800, 2900, 3001, 3002, 3003, 3101, 3102, 3103,
  3104, 3200, 3301, 3302, 3401, 3402, 3501, 3502, 3601, 3602,
  3701, 3702, 3800, 3901, 3902, 4001, 4002, 4100, 4201, 4202,
  4203, 4204, 4205, 4301, 4302, 4303, 4400, 4501, 4502, 4601,
  4602, 4701, 4702, 4801, 4802, 4803, 4901, 4902, 4903, 4904,
  5001, 5002, 5003, 5004, 5005, 5006, 5007, 5008, 5009, 5201,
  5202, 5400);

  NAD83: array[1..134 {134}] of Integer = (101, 102,
    5010, 5300,  201,  202,  203,  301,  302,  401,  402,  403,
     404, 405,   406, 0000,  501,  502,  503,  600,  700,  901,
     902,  903, 1001, 1002, 5101, 5102, 5103, 5104, 5105, 1101,
    1102, 1103, 1201, 1202, 1301, 1302, 1401, 1402, 1501, 1502,
    1601, 1602, 1701, 1702, 1703, 1801, 1802, 1900, 2001, 2002,
    2101, 2102, 2103, 2111, 2112, 2113, 2201, 2202, 2203, 2301,
    2302, 2401, 2402, 2403, 2500, 0000, 0000, 2600, 0000, 2701,
    2702, 2703, 2800, 2900, 3001, 3002, 3003, 3101, 3102, 3103,
    3104, 3200, 3301, 3302, 3401, 3402, 3501, 3502, 3601, 3602,
    3701, 3702, 3800, 3900, 0000, 4001, 4002, 4100, 4201, 4202,
    4203, 4204, 4205, 4301, 4302, 4303, 4400, 4501, 4502, 4601,
    4602, 4701, 4702, 4801, 4802, 4803, 4901, 4902, 4903, 4904,
    5001, 5002, 5003, 5004, 5005, 5006, 5007, 5008, 5009, 5200,
    0000, 5400);




var

{FROM SPHDZ static double}
{  major: array[0..19 ] of Double =
  ( 6378206.4,  6378249.145, 6377397.155,  6378157.5,
    6378388.0,  6378135.0,   6377276.3452, 6378145.0,
    6378137.0,  6377563.396, 6377304.063,  6377340.189,
    6378137.0,  6378155.0,   6378160.0,    6378245.0,
    6378270.0,  6378166.0,   6378150.0,    6370997.0);}
major: array[0..30{SPHDCT-1}] of Double =(
		6378206.4,		// 0: Clarke 1866 (default) */
		6378249.145,		// 1: Clarke 1880 */
		6377397.155,		// 2: Bessel */
		6378157.5,		// 3: International 1967 */
		6378388.0,		// 4: International 1909 */
		6378135.0,		// 5: WGS 72 */
		6377276.3452,		// 6: Everest */
		6378145.0,		// 7: WGS 66 */
    6378137.0,		// 8: GRS 1980 */
		6377563.396,		// 9: Airy */
		6377304.063,		// 10: Modified Everest */
		6377340.189,		// 11: Modified Airy */
    6378137.0,		// 12: WGS 84 */
		6378155.0,		// 13: Southeast Asia */
		6378160.0,		// 14: Australian National */
		6378245.0,		// 15: Krassovsky */
    6378270.0,		// 16: Hough */
		6378166.0,		// 17: Mercury 1960 */
		6378150.0,		// 18: Modified Mercury 1968 */
		6370997.0,		// 19: Sphere of Radius 6370997 meters*/
		6377483.865,		// 20: Bessel 1841(Namibia) */
		6377298.556,		// 21: Everest (Sabah & Sarawak) */
		6377301.243,		// 22: Everest (India 1956) */
		6377295.664,		// 23: Everest (Malaysia 1969) */
		6377304.063,		// 24: Everest (Malay & Singapr 1948)*/
		6377309.613,		// 25: Everest (Pakistan) */
		6378388.0,		// 26: Hayford */
		6378200.0,		// 27: Helmert 1906 */
		6378160.000,		// 28: Indonesian 1974 */
		6378160.0,		// 29: South American 1969 */
		6378165.0);		// 30: WGS 60 */

{static double}
  minor: array[0..30]of Double = (
{   6356583.8,      6356514.86955,  6356078.96284, 6356772.2,
    6356911.94613,  6356750.519915, 6356075.4133,  6356759.769356,
    6356752.31414,  6356256.91,     6356103.039,   6356034.448,
    6356752.314245, 6356773.3205,   6356774.719,   6356863.0188,
    6356794.343479, 6356784.283666, 6356768.337303,6370997.0);}
		6356583.8,		// 0: Clarke 1866 (default) */
		6356514.86955,		// 1: Clarke 1880 */
		6356078.96284,		// 2: Bessel */
		6356772.2,		// 3: International 1967 */
    6356911.94613,		// 4: International 1909 */
		6356750.519915,		// 5: WGS 72 */
		6356075.4133,		// 6: Everest */
    6356759.769356,		// 7: WGS 66 */
		6356752.31414,		// 8: GRS 1980 */
		6356256.91,		// 9: Airy */
    6356103.039,		// 10: Modified Everest */
		6356034.448,		// 11: Modified Airy */
		6356752.314245,		// 12: WGS 84 */
    6356773.3205,		// 13: Southeast Asia */
		6356774.719,		// 14: Australian National */
		6356863.0188,		// 15: Krassovsky */
    6356794.343479,		// 16: Hough */
		6356784.283666,		// 17: Mercury 1960 */
		6356768.337303,		// 18: Modified Mercury 1968 */
    6370997.0,		// 19: Sphere of Radius 6370997 meters*/
		6356165.382966,		// 20: Bessel 1841(Namibia) */
		6356097.571445,		// 21: Everest (Sabah & Sarawak) */
		6356100.228368,		// 22: Everest (India 1956) */
		6356094.667915,		// 23: Everest (Malaysia 1969) */
		6356103.038993,		// 24: Everest (Malay & Singapr 1948)*/
		6356108.570542,		// 25: Everest (Pakistan) */
		6356911.946128,		// 26: Hayford */
		6356818.169,		// 27: Helmert 1906 */
		6356774.504086,		// 28: Indonesian 1974 */
		6356774.719,		// 29: South American 1969 */
		6356783.287);		// 30: WGS 60 */




{PURPOSE:  The following functions are included in REPORT.C
    INIT:Initializes the output device for error messages and report headings.
    P_ERROR:Reports errors to the terminal, a specified file, or both.
    PTITLE, RADIUS, RADIUS2, CENLON, CENLONMER, CENLAT, ORIGIN,
    STANPARL, STPARL1, OFFSET, GENRPT, GENRPT_LONG,
 PBLANK:Reports projection parameters to the terminal,specified file, or both.
}
{procedure PError(what, where: string);}
procedure DoProjError(What,i,i2,i3: Integer;iD:Double);
{function Init(ipr, jpr: Integer; efile, pfile: string): Integer;
procedure close_file; }
procedure Reported(A: string);
procedure ptitle(A: string);
procedure radius(A: double);
procedure radius2(A, B: double);
procedure cenlon(A: double);
procedure cenlonmer(A: double);
procedure cenlat(A: double);
procedure origin(A: double);
procedure stanparl(A, B: double);
procedure stparl1(A: double);
procedure offsetp(A, B: double);
procedure genrpt(A: double; S: string);
procedure GenrptLong(A: Integer; S: string);
procedure pblank;

{PURPOSE:  The following functions are included in CPROJ.C.}
function asinz(con: double): double;
  {double asinz (double con);
  ASINZ:	  Eliminates roundoff errors.}
function msfnz(eccent, sinphi, cosphi: double): double;
{double msfnz (double eccent, double sinphi, double cosphi);
  MSFNZ:	  Computes the constant small m for Oblique Equal Area.}
function qsfnz(eccent, sinphi, cosphi: double): double;
{double qsfnz (double eccent, double sinphi, double cosphi);
  QSFNZ:	  Computes the constant small q for Oblique Equal Area.}
function phi1z(eccent, qs: double; var flag: Integer): double;
{double phi1z (double eccent, double qs, long int *flag);
  PHI1Z:	  Computes phi1 for Albers Conical Equal-Area.}
function phi2z(eccent, ts: double; var flag: Integer): double;
{double phi2z(double eccent, double ts, long int *flag);
  PHI2Z:	  Computes the latitude angle, phi2, for Lambert
     Conformal Conic and Polar Stereographic.... Mercator too}
function phi3z(ml, e0, e1, e2, e3: double; var flag: Integer):  double;
{double phi3z(double ml, double e0, double e1, double e2, double e3, long int *flag);
  PHI3Z:	  Computes the latitude, phi3, for Equidistant Conic.}

function phi4z(eccent, e0, e1, e2, e3, a, b: double; var c, phi: double): double;
{double phi4z (double eccent, double e0, double e1, double e2, double e3,
              double a, double b, double *c, double *phi);
  PHI4Z:	  Computes the latitude, phi4, for Polyconic.              }
function pakcz(pak: double): double;
{double pakcz(double pak);
  PAKCZ:	  Converts a 2 digit alternate packed DMS format to
     standard packed DMS format.}
function pakr2dm(pak: double): double;
{double pakr2dm(double pak);
  PAKR2DM:  Converts radians to 3 digit packed DMS format.}
(*double paksz(double ang, long int *iflg)
// angle which in DMS    error flag number *)
function Paksz(ang: double; var iflg: Integer): Double;
{function paksz(ang,iflg: double): double;}

function tsfnz(eccent, phi, sinphi: double): double;
{double tsfnz(double eccent, double phi, double sinphi);
  TSFNZ:	  Computes the small t for Lambert Conformal Conic and
     Polar Stereographic.}
function sign(x: double): Integer;
{int sign(double x);
  SIGN:	  Returns the sign of an argument.}
function AdjustLon(x: double): double;
{double adjust_lon(double x);
  ADJUST_LON:  Adjusts a longitude angle to range -180 to 180.}
function AdjustLonRadians(x: double): double;
  // Angle in radians
function e0fn(x: double): double;
{double e0fn(double x);}
function e1fn(x: double): double;
{double e1fn(double x);}
function e2fn(x: double): double;
{double e2fn(double x);}
function e3fn(x: double): double;
{double e3fn(double x);}
{  E0FN, E1FN, E2FN, E3FN:
     Computes the constants e0,e1,e2,and e3 for
     calculating the distance along a meridian.}
function e4fn(x: double): double;
{double e4fn(double x);
  E4FN:	  Computes e4 used for Polar Stereographic.}
function mlfn(e0, e1, e2, e3, phi: double): double;
{double mlfn(double e0, double e1, double e2, double e3, double phi);
  MLFN:	  Computes M, the distance along a meridian.}

function CalcUtmZone(lon: double): Integer;
{long calc_utm_zone(double lon);
  CALC_UTM_ZONE:	Calculates the UTM zone number.}
  // Function to calculate UTM zone number--
  //  NOTE Longitude entered in DEGREES!!!
{Procedure sincos(val: double; var sin_val, cos_val: double);}
{  SINCOS:	  Calculates the sine and cosine.}

(* Misc macros   ----------- Cproj.h
Function SQUARE(x)       x * x   /* x**2 */
#define CUBE(x)     x * x * x   /* x**3 */
#define QUAD(x) x * x * x * x   /* x**4 */
#define GMAX(A, B)      ((A) > (B) ? (A) : (B)) /* assign maximum of a and b */
#define GMIN(A, B)      ((A) < (B) ? (A) : (B)) /* assign minimum of a and b */
#define IMOD(A, B)      (A) - (((A) / (B)) * (B)) /* Integer mod function  Result:=A mod B */
*)

    // Helpful functions for getting
    //the string representations of units and datums
    // from Projection.cpp
    function ProjectiontoString(todoProjection :Integer): string;
    function datumToString( todoDATUM :Integer ):string;
    function unitToString( todoUNIT  :Integer ):string;
    function packedDMSToDouble(  packedDMS:double ):double;
{    function packedDMSToString(  packedDMS:double;  bLat:boolean ):string;}

implementation

uses buzMemoFrm;

procedure DoProjError(What,i,i2,i3: Integer;iD:Double);
var  { mtError a red stop sign.}
{  s:array[0..79] of Char;}
  MessageHdr,Messagestr: string;
{  HelpContextl: Longint;  }
begin
{  HelpContextl := 0;}
  case What of
-1: Messagestr :='User defined';
  0:MessageHdr :='INFORMATIONAL';
1: MessageHdr :='phi1z-conv';
2: MessageHdr :='phi2z-conv';
3: MessageHdr :='PHI3Z-CONV';
4: MessageHdr :='phi4z-conv';
5: MessageHdr :='untfz-unit';
7: MessageHdr :='File Functions';
11: MessageHdr :='utm-forint';
12: MessageHdr :='utm-invint';
21: MessageHdr :='state-init';
31: MessageHdr :='alber-forinit';
32: MessageHdr :='alber-invinit';
41: MessageHdr :='lamcc-for';
42: MessageHdr :='lamcc-inv';
53: MessageHdr :='mer-forward';
81: MessageHdr :='eqcon_for';
93: MessageHdr :='tm-for';
95: MessageHdr :='TM-INVERSE';
103: MessageHdr :='ster-for';
113: MessageHdr :='lamaz-forward';
115: MessageHdr :='lamaz-inverse';
123: MessageHdr :='azim-for';
125: MessageHdr :='azim-inv';
133: MessageHdr :='gnomfor-conv';
143: MessageHdr :='orth-for';
145: MessageHdr :='orth-inv';
153: MessageHdr :='gvnsp-for';
155: MessageHdr :='gvnsp-for';
164: MessageHdr :='sinusoidal-inverse';
174: MessageHdr :='equi-inv';
201: MessageHdr :='omer-init';
202: MessageHdr :='omer-init..omerforint..omerinvint';
205: MessageHdr :='omer-for';
214: MessageHdr :='som-forward';
215: MessageHdr :='som-inverse';
234: MessageHdr :='robinv-conv';
235: MessageHdr :='alcon-inv';
241: MessageHdr :='IntMoll-forward';
242: MessageHdr :='Mollweide-forward';
243: MessageHdr :='Wagneriv-Forward';
251: MessageHdr :='goode-forward';
252: MessageHdr :='goode-inverse';
1101: MessageHdr :='untfz-code';
1116 : MessageHdr :='paksz-deg';
1117 : MessageHdr :='paksz-min';
1118 : MessageHdr :='paksz-sec';
2002 : MessageHdr :='Datum Conversion';
  End;
  case i of
-1: Messagestr :='Unsupported format';
0: Messagestr :=	'Invalid spheroid selection'
          +#13#10+'Reset to 0';
2: Messagestr :=	'Convergence error';
3: Messagestr :=	'Latitude failed to converge after 15 iterations';
4: Messagestr :=	'Latitude failed to converge';
5: Messagestr :=	'Illegal source or target unit code';
6: Messagestr :=	'Output file name not specified';
7: Messagestr :=	'Read Write file Error';
11: Messagestr :=	'Illegal zone number';
21: Messagestr :=	'Illegal zone #'
                  +Inttostr(i2)+{<zone_code>}
                  ' for spheroid # '
                  +Inttostr(i3){<sph_code>};
22: Messagestr :=	'Error opening State Plane parameter file';
23: Messagestr :=	'Illegal spheroid # '+Inttostr(i2){<sph_code>};
31: Messagestr :=	'Equal latitudes for St. Parallels'
                  +#13#10+'on opposite sides of equator';
41: Messagestr :=	'Equal Latitudes for St. Parallels'
                  +#13#10+'on opposite sides of equator';
44: Messagestr :=	'Point can not be projected';
53: Messagestr :=	'Transformation cannot be computed at the poles';
81: Messagestr :=	'Standard Parallels on opposite sides of equator';
93: Messagestr :=	'Point projects into infinity';
95: Messagestr :=	'Latitude failed to converge';
103: Messagestr := 'Point projects into infinity';
113: Messagestr :=	'Point projects to a circle of radius = '+Inttostr(i2){ <radius>};
115: Messagestr :=	'Input data error';
123: Messagestr :=	'Point projects into a circle of radius = '+Floattostr(iD){ <radius>};
125: Messagestr :=	'Input data error';
133: Messagestr :=	'Point projects into infinity';
143: Messagestr :=	'Point can not be projected';
145: Messagestr :=	'Input data error';
153: Messagestr :=	'Point cannot be projected';
155: Messagestr :=	'Input data error';
164: Messagestr :=	'Input data error';
174: Messagestr :=	'Input data error';
201: Messagestr :=	'Input data error';
202: Messagestr :=	'Input data error';
205: Messagestr :=	'Point projects into infinity';
214: Messagestr :=	'50 iterations without convergence';
234: Messagestr :=	'Too many iterations in inverse';
235: Messagestr :=	'Too many iterations in inverse';
236: Messagestr :=	'Too many iterations in inverse';
241: Messagestr :=	'Iteration failed to converge';
251: Messagestr :=	'Iteration failed to converge';
252: Messagestr :=	'Input data error';
1101: Messagestr :=	'Incompatable unit codes'+#13#10+
                     unitToString(i2) + '  '+unitToString(i3);
1116: Messagestr :=	'Illegal DMS field';
2002: Messagestr :=	'FATAL ERROR: '+Inttostr(i2);
  end;
buzMemoForm.Show;
buzMemoForm.ProLogMemo.Lines.Append(MessageHdr);
buzMemoForm.ProLogMemo.Lines.Append(Messagestr);

         { Beep(10000, 1000);}
       { if ((ErrorBeepOn) and (not VoicesON)) then Beep;
        if ((ErrorBeepOn) and (VoicesON)
          and (FileExists(ExtractFilePath(ParamStr(0))
                          + 'ERRORV.WAV')) )then
        begin
          StrPCopy(s, ExtractFilePath(ParamStr(0)) + 'ERRORV.WAV');
          if (MMSysHandle <> 0) then  PlaySound(s,  0);
        end;}
{mbYesNoCancel,}
{   MessageDlgPos(MessageHdr+#13#10+Messagestr, mtError,[mbOK],
          buzHelpContextl, buzMessageX, buzMessageY);}
end;

//  from report Function to report errors
//  void p_error(char *what, char *where)
//  initialize output device
//  int init(long int ipr, long int jpr, char *efile, char *pfile)
//  flag for printing errors (0,1,or 2)
//  flag for printing parameters (0,1,or 2)
//  name of error file
//  name of parameter file
//  Omitted due to rewrite... use Memo and Defines
//  Functions to report projection parameters

procedure Reported(A: string);
begin
  buzMemoForm.ProLogMemo.Lines.Append(A);
End;

procedure ptitle(A: string);
begin
Reported('PROJECTION PARAMETERS:');
Reported(A);
{  MapForm.ProLogMemo.Lines.Append('PROJECTION PARAMETERS:');
  MapForm.ProLogMemo.Lines.Append(A);}
end;

procedure radius(A: double);
begin
Reported('Radius of Sphere: '+Floattostr(A));
{  MapForm.ProLogMemo.Lines.Append('Radius of Sphere: '+Floattostr(A));}
end;

procedure radius2(A, B: double);
begin
  {fprintf(fptr_p,"   Semi-Minor Axis of Ellipsoid:     %lf meters\n",B);}
Reported('Semi-Major Axis of Ellipsoid: meters\'+Floattostr(A));
{  MapForm.ProLogMemo.Lines.Append('Semi-Major Axis of Ellipsoid: meters\'+Floattostr(A));}
Reported('Semi-Minor Axis of Ellipsoid: meters\'+Floattostr(B));
{  MapForm.ProLogMemo.Lines.Append('Semi-Minor Axis of Ellipsoid: meters\'+Floattostr(B));}
end;

procedure cenlon(A: double);
begin
  {fprintf(fptr_p,"   Longitude of Center:     %lf degrees\n",A*R2D);}
Reported('Longitude of Center: degrees\'+Floattostr(A*R2D));
{  MapForm.ProLogMemo.Lines.Append('Longitude of Center: degrees\'+Floattostr(A*R2D));}
end;

procedure cenlonmer(A: double);
begin
  {printf("   Longitude of Central Meridian:     %lf degrees\n",A*R2D);}
Reported('Longitude of Central Meridian: degrees\'+Floattostr(A*R2D));
{  MapForm.ProLogMemo.Lines.Append('Longitude of Central Meridian: degrees\'+Floattostr(A*R2D));}
end;

procedure cenlat(A: double);
begin
  {printf("   Latitude  of Center:     %lf degrees\n",A*R2D);}
Reported('Latitude of Center: degrees\'+Floattostr(A*R2D));
{  MapForm.ProLogMemo.Lines.Append('Latitude of Center: degrees\'+Floattostr(A*R2D));}
end;

procedure origin(A: double);
begin
  {printf("   Latitude of Origin:     %lf degrees\n",A*R2D);}
Reported('Latitude  of Origin: degrees\'+Floattostr(A*R2D));
{  MapForm.ProLogMemo.Lines.Append('Latitude  of Origin: degrees\'+Floattostr(A*R2D));}
end;

procedure stanparl(A, B: double);
begin
  {printf("   1st Standard Parallel:     %lf degrees\n",A*R2D);}
Reported('1st Standard Parallel: degrees\'+Floattostr(A*R2D));
{  MapForm.ProLogMemo.Lines.Append('1st Standard Parallel: degrees\'+Floattostr(A*R2D));}
Reported('2nd Standard Parallel: degrees\'+Floattostr(B*R2D));
{  MapForm.ProLogMemo.Lines.Append('2nd Standard Parallel: degrees\'+Floattostr(B*R2D));}
end;

procedure stparl1(A: double);
begin
  {printf("   Standard Parallel:     %lf degrees\n",A*R2D);}
  Reported('Standard Parallel: degrees\'+Floattostr(A*R2D));
{  MapForm.ProLogMemo.Lines.Append('Standard Parallel: degrees\'+Floattostr(A*R2D));}
end;

procedure offsetp(A, B: double);
begin
  {fprintf(fptr_p,"   False Easting:      %lf meters \n",A);}
  Reported('False Easting: meters\'+Floattostr(A));
{  MapForm.ProLogMemo.Lines.Append('False Easting: meters\'+Floattostr(A));}
  Reported('False Northing: meters\'+Floattostr(B));
{  MapForm.ProLogMemo.Lines.Append('False Northing: meters\'+Floattostr(B));}
end;

procedure genrpt(A: double; S: string);
begin {
  {printf("   %s %lf\n", S, A);}
  Reported(S+Floattostr(A));
{  MapForm.ProLogMemo.Lines.Append(S+Floattostr(A));}
end;

procedure GenrptLong(A: Integer; S: string);
begin
  {printf("   %s %d\n", S, A);}
  Reported(S+Floattostr(A));
{  MapForm.ProLogMemo.Lines.Append(S+Floattostr(A));}
end;

procedure pblank;
begin
  {fprintf(fptr_p,"\n");}
  Reported(#13#10);
{  MapForm.ProLogMemo.Lines.Append(#13#10);}
end;


{from CPROJ}
//* Function to eliminate roundoff errors in asin
//----------------------------------------------*/
function asinz(con: double): double;
begin
  if (abs(con) > 1.0) then
  begin
    if (con > 1.0) then con := 1.0
    else con := -1.0;
  end;
  result := (arcsin(con));
{ ArcSin returns the inverse sine of X.
 X must be between -1 and 1.
 The return value will be in the range [-Pi/2..Pi/2], in radians.}
end;

//* Function to compute the constant small m which is the radius of
//   a parallel of latitude, phi, divided by the semimajor axis.
//---------------------------------------------------------------*/
function msfnz(eccent, sinphi, cosphi: double): double;
var con: double;
begin
  con := eccent * sinphi;
  result := ((cosphi / (sqrt(1.0 - con * con))));
end;

//* Function to compute constant small q which is the radius of a
//   parallel of latitude, phi, divided by the semimajor axis.
//------------------------------------------------------------*/
function qsfnz(eccent, sinphi, cosphi: double): double;
var con: double;
begin
  if (eccent > 1.0E-7) then
  begin
    con := eccent * sinphi;
    result := ((1.0 - eccent * eccent)
      * (sinphi / (1.0 - con * con) - (0.5 / eccent) *
      Ln((1.0 - con) / (1.0 + con))));
{e := Exp(1.0);
Ln returns the natural logarithm (Ln(e) = 1)
of the real-type expression X.}
  end
  else result := (2.0 * sinphi);
end;

//* Function to compute phi1, the latitude for the inverse of the
//   Albers Conical Equal-Area projection.
//-------------------------------------------*/
function phi1z(eccent, qs: double; var flag: Integer): double;
//* Eccentricity angle in radians		*/
//* Angle in radians				*/
//* Error flag number				*/
var
  eccnts, dphi, con, com, sinpi, cospi, phi: Double;
{double asinz(double con);}
  i: Integer;
begin
  phi := asinz(0.5 * qs);
  if (eccent < EPSLN) then
  begin
    result := (phi);
    exit;
  end;
  eccnts := eccent * eccent;
  for i := 1 to 25 do
  begin
    sinpi := Sin(phi);
    cospi := Cos(phi);
    {sincos(phi, sinpi, cospi); }
    con := eccent * sinpi;
    com := 1.0 - con * con;
    dphi := 0.5 * com * com / cospi * (qs / (1.0 - eccnts) - sinpi /
      com +
      0.5 / eccent * ln((1.0 - con) / (1.0 + con)));
    phi := phi + dphi;
    if (abs(dphi) <= 1E-7) then
    begin
      result := (phi);
      exit;
    end;
  end;
{  PError('Convergence error', 'phi1z-conv');}
  DoProjError(1,2,0,0,0);
  flag := 001;
  result := (ERROR);
end;

//* Function to compute the latitude angle, phi2, for the inverse of the
//   Lambert Conformal Conic and Polar Stereographic projections.
//----------------------------------------------------------------*/
function phi2z(eccent, ts: double; var flag: Integer): double;
//* Spheroid eccentricity		*/
//* Constant value t			*/
//* Error flag number			*/
var
  eccnth, phi, con, dphi, sinpi: double;
  i: Integer;
begin
  flag := 0;
  eccnth := 0.5 * eccent;
  phi := HALF_PI - 2 * arctan(ts);
  for i := 0 to 15 do
  begin
    sinpi := sin(phi);
    con := eccent * sinpi;
    dphi := HALF_PI -
            2 * arctan(ts * (power( ((1.0 - con)
                                   / (1.0 + con)), eccnth) ) )
           - phi;
    phi := phi + dphi;
    if (abs(dphi) <= 0.0000000001) then
    begin
      result := (phi);
      exit;
    end;
  end;
{  PError('Convergence error', 'phi2z-conv');}
  DoProjError(2,2,0,0,0);
  flag := 002;
  result := (002);
end;

// Function to compute latitude, phi3,
//for the inverse of the Equidistant Conic projection.
//-----------------------------------------------------------------
function phi3z(ml, e0, e1, e2, e3: double; var flag: Integer): double;
// Constant, Constant, Constant, Constant,Constant	, Error flag number
var
  phi, dphi: double;
  i: Integer;
begin
  phi := ml;
  for i := 0 to 15 do
  begin
    dphi := (ml + e1 * sin(2.0 * phi) - e2 * sin(4.0 * phi) + e3 *
      sin(6.0 * phi))
      / e0 - phi;
    phi := phi + dphi;
    if (abs(dphi) <= 0.0000000001) then
    begin
      flag := 0;
      result := (phi);
      exit;
    end;
  end;
{  PError('Latitude failed to converge after 15 iterations',
    'PHI3Z-CONV');}
  DoProjError(3,3,0,0,0);
  flag := 3;
  result := (3);
end;

// Function to compute, phi4,
//the latitude for the inverse of the Polyconic projection.
//------------------------------------------------------------
function phi4z(eccent, e0, e1, e2, e3, a, b: double;
               var c, phi: double): double;
// Spheroid eccentricity squared
var
  sinphi, sin2ph, tanphi, ml, mlp, con1, con2, con3, dphi: double;
  i: Integer;
begin
  phi := a;
  for i := 1 to 15 do
  begin
    sinphi := sin(phi);
    tanphi := tan(phi);
    c := tanphi * sqrt(1.0 - eccent * sinphi * sinphi);
    sin2ph := sin(2.0 * phi);
{        ml = e0 * *phi - e1 * sin2ph + e2 * sin (4.0 *  *phi);
        mlp = e0 - 2.0 * e1 * cos (2.0 *  *phi) + 4.0 * e2 *
              cos (4.0 *  *phi);}

    ml := e0 * phi - e1 * sin2ph + e2 * sin(4.0 * phi) - e3 *
      sin(6.0 * phi);
    mlp := e0 - 2.0 * e1 * cos(2.0 * phi) + 4.0 * e2 *
      cos(4.0 * phi) - 6.0 * e3 * cos(6.0 * phi);
    con1 := 2.0 * ml + c * (ml * ml + b) - 2.0 * a * (c * ml + 1.0);
    con2 := eccent * sin2ph * (ml * ml + b - 2.0 * a * ml) / (2.0 *
      c);
    con3 := 2.0 * (a - ml) * (c * mlp - 2.0 / sin2ph) - 2.0 * mlp;
    dphi := con1 / (con2 + con3);
    phi := phi + dphi;
    if (abs(dphi) <= 0.0000000001) then begin
      result := (OK);
      exit;
    end;
  end;
{  PError('Lattitude failed to converge', 'phi4z-conv'); }
  DoProjError(4,4,0,0,0);
  result := (004);
end;

// Function to convert 2 digit alternate packed DMS format (+/-)DDDMMSS.SSS
//   to 3 digit standard packed DMS format (+/-)DDDMMMSSS.SSS.
//-----------------------------------------------------------------
function pakcz(pak: double): double;
// Angle in alternate packed DMS format
var
  con, secs: double;
  degs, mins: Integer;
  sgna: char;
begin
  sgna := ' ';
  if (pak < 0.0) then sgna := '-';
  con := abs(pak);
  degs := Round((con / 10000.0) + 0.001);
  con := con - degs * 10000;
  mins := Round((con / 100.0) + 0.001);
  secs := con - mins * 100;
  con := (degs) * 1000000.0 + (mins) * 1000.0 + secs;
  if (sgna = '-') then con := -con;
  result := (con);
end;

// Function to convert radians to
// 3 digit packed DMS format (+/-)DDDMMMSSS.SSS
//---------------------------------------------------------
function pakr2dm(pak: double): double;
// Angle in radians
var
  con, secs: double;
  degs, mins: Integer;
  sgna: char;
begin
  sgna := ' ';
  pak := pak * R2D;
  if (pak < 0.0) then sgna := '-';
  con := abs(pak);
  degs := round(con);
  con := (con - degs) * 60;
  mins := round(con);
  secs := (con - mins) * 60;
  con := ((degs) * 1000000.0 + (mins) * 1000.0 + secs);
  if (sgna = '-') then con := -con;
  result := (con);
end;


{function paksz(ang,iflg: double): double;
PURPOSE:	This function converts a packed DMS angle to seconds.  The
		standard packed DMS format is:
		degrees * 1000000 + minutes * 1000 + seconds
		Example:	ang = 120025045.25 yields
				deg = 120
				min = 25
				sec = 45.25
		The algorithm used for the conversion is as follows:
		1.  The absolute value of the angle is used.
		2.  The degrees are separated out:
		    deg = ang/1000000 	(fractional portion truncated)
		3.  The minutes are separated out:
		    min = (ang - deg * 1000000) / 1000 	   (fractional
							portion	truncated)
		4.  The seconds are then computed:
		    sec = ang - deg * 1000000 - min * 1000
		5.  The total angle in seconds is computed:
		    sec = deg * 3600.0 + min * 60.0 + sec
		6.  The sign of sec is set to that of the input angle.}
(*
function paksz(ang,iflg: double): double;
{double ang;		 angle which in DMS
long *iflg;	error flag number}
var
 fac,		// sign flag
 deg,		// degree variable
 min,		// minute variable
 sec,		// seconds variable
 tmp:Double;		// temporary variable
 i:Integer;		 // variable
Begin
{iflg := 0;}
if (ang < 0.0)then fac := -1 else   fac := 1;
// find degrees
sec := abs(ang);
tmp := 1000000.0;
i := Round( sec/tmp);
if (i > 360)then
  begin
{  p_error("Illegal DMS field","paksz-deg"); }
  iflg := 1116;
    DoProjError(1116,1116,0,0,iflg);
  Result:=(ERROR);
  exit;
  end
else
  deg := i;

// find minutes
sec := sec - deg * tmp;
tmp := 1000;
i := Round( sec / tmp);
if (i > 60)then
  begin
{  p_error("Illegal DMS field","paksz-min");}
  iflg := 1116;
    DoProjError(1116,1116,0,0,iflg);
  Result:=(ERROR);
  exit;
  end else  min := i;

// find seconds
sec := sec - min * tmp;
if (sec > 60)then
  begin
{  p_error("Illegal DMS field","paksz-sec");}
  iflg := 1116;
  DoProjError(1116,1116,0,0,iflg);
  Result:=(ERROR);
  exit;
  end else  sec := fac * (deg * 3600.0 + min * 60.0 + sec);
deg := sec / 3600.0;

Result:=(deg);
End;
*)

(* double paksz(double ang, long int *iflg)
             /* angle which in DMS		*/
             /* error flag number		*/
NAME                            PAKSZ
PURPOSE:	This function converts a packed DMS angle to seconds.
                The standard packed DMS format is:
  degrees * 1000000 + minutes * 1000 + seconds
  Example:	ang = 120025045.25 yields
    deg = 120
    min = 25
    sec = 45.25
  The algorithm used for the conversion is as follows:
  1.  The absolute value of the angle is used.
  2.  The degrees are separated out:
      deg = ang/1000000 	(fractional portion truncated)
  3.  The minutes are separated out:
      min = (ang - deg * 1000000) / 1000 	   (fractional
       portion	truncated)
  4.  The seconds are then computed:
      sec = ang - deg * 1000000 - min * 1000
  5.  The total angle in seconds is computed:
      sec = deg * 3600.0 + min * 60.0 + sec
  6.  The sign of sec is set to that of the input angle.
*)

function Paksz(ang: double; var iflg: Integer): Double;
var
  fac, //* sign flag			*/
  deg, //* degree variable		*/
  min, //* minute variable		*/
  sec, //* seconds variable		*/
  tmp: double; //* temporary variable		*/
  i: Integer; //* temporary variable		*/
begin
  iflg := 0;
  if (ang < 0.0) then fac := -1 else fac := 1;

//* find degrees-------------*/
  sec := abs(ang);
  tmp := 1000000.0;
  i := round(sec / tmp);
  if (i > 360) then
  begin
    {PError('Illegal DMS field', 'paksz-deg');}
    DoProjError(1116,1116,0,0,0);
    iflg := 1116;
    result := (ERROR);
    Exit;
  end
  else deg := i;

//* find minutes-------------*/
  sec := sec - (deg * tmp);
  tmp := 1000;
  i := round(sec / tmp);
  if (i > 60) then
  begin
    {PError('Illegal DMS field', 'paksz-min');  }
    DoProjError(1117,1116,0,0,0);
    iflg := 1116;
    result := (ERROR);
    Exit;
  end else min := i;

//* find seconds-------------*/
  sec := sec - (min * tmp);
  if (sec > 60) then
  begin
   { PError('Illegal DMS field', 'paksz-sec');}
    DoProjError(1118,1116,0,0,0);
    iflg := 1116;
    result := (ERROR);
    Exit;
  end
  else sec := fac * (deg * 3600.0 + min * 60.0 + sec);
  deg := sec / 3600.0;
  result := (deg);
end;
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// Function to compute the constant small t for use in the forward
//   computations in the Lambert Conformal Conic and the Polar
//   Stereographic projections.
//--------------------------------------------------------------
function tsfnz(eccent, phi, sinphi: double): double;
// Eccentricity of the spheroid
// Latitude phi
// Sine of the latitude
var
  con, com: double;
begin
  con := eccent * sinphi;
  com := 0.5 * eccent;
  con := power(((1.0 - con) / (1.0 + con)), com);
  result := (tan(0.5 * (HALF_PI - phi)) / con);
end;


//* Function to return the sign of an argument
//  ------------------------------------------*/
function sign(x: double): Integer;
begin
  if (x < 0.0) then result := (-1) else result := (1);
end;

// Function to adjust a longitude angle to range from
// -180 to 180 radians   added if statments
//  --------------------------------------------------------
// Modified by Mike Childs - 4/21/98
function AdjustLon(x: double): double;
begin
{    // Check to see if x is already OK}
  if (abs(x) <= PI) then result := x
  else
  begin
    x := x - sign(x) * Round(x / (2 * PI)) * 2 * PI;
    if (x > PI) then x := x - (2 * PI);
    result := x;
  end;
end;

function AdjustLonRadians(x: double): double;
// Angle in radians
var count: Integer;
begin
  for count := 0 to MAX_VAL do
  begin
    if (abs(x) <= PI) then break
    else
      if ((abs(x / PI)) < 2) then
        x := x - (sign(x) * TWO_PI)
      else
        if ((abs(x / TWO_PI)) < MAXLONG) then
        begin
          x := x - (((x / TWO_PI)) * TWO_PI);
        end
        else
          if ((abs(x / (MAXLONG * TWO_PI))) < MAXLONG) then
          begin
            x := x - (((x / (MAXLONG * TWO_PI)))
                      * (TWO_PI * MAXLONG));
          end
          else
            if ((abs(x / (DBLLONG * TWO_PI))) < MAXLONG) then
            begin
              x := x - (((x / (DBLLONG * TWO_PI)))
                        * (TWO_PI * DBLLONG));
            end
            else
              x := x - (sign(x) * TWO_PI);
  end;
  result := x;
end;

//* Functions to compute the constants e0, e1, e2, and e3 which are used
//   in a series for calculating the distance along a meridian.  The
//   input x represents the eccentricity squared.
//----------------------------------------------------------------*/

function e0fn(x: double): double;
begin result := (1.0 - 0.25 * x * (1.0 + x / 16.0 * (3.0 + 1.25 *
  x))); end;

function e1fn(x: double): double;
begin result := (0.375 * x * (1.0 + 0.25 * x * (1.0 + 0.46875 * x)));
  end;

function e2fn(x: double): double;
begin result := (0.05859375 * x * x * (1.0 + 0.75 * x)); end;

function e3fn(x: double): double;
begin result := (x * x * x * (35.0 / 3072.0)); end;

//* Function to compute the constant e4 from the input of the eccentricity
//   of the spheroid, x.  This constant is used in the Polar Stereographic
//   projection.
//--------------------------------------------------------------------*/
function e4fn(x: double): double;
var
  con, com: double;
begin
  con := 1.0 + x;
  com := 1.0 - x;
  Result := (sqrt((power(con, con)) * (power(com, com))));
end;

// Function computes the value of M which is the distance along a meridian
//   from the Equator to latitude phi.
//------------------------------------------------
function mlfn(e0, e1, e2, e3, phi: double): double;
begin
  Result := (e0 * phi - e1 * sin(2.0 * phi) + e2 * sin(4.0 * phi) - e3
    * sin(6.0 * phi));
end;

// Function to calculate UTM zone number--
//  NOTE Longitude entered in DEGREES!!!
//  ---------------------------------------------------------------------------*/
function CalcUtmZone(lon: double): Integer;
begin
  Result := Round((((lon + 180.0) / 6.0) + 1.0));
end;

(*/* Not part of the original gctpc distribution. */
#ifdef NEED_SINCOS
void
sincos(double val, double *sin_val, double *cos_val);
procedure SinCos(Theta: Extended; var Sin, Cos: Extended); register;
Description
Call SinCos to obtain the sine and cosine
of an angle expressed in radians.
 Theta specifies the angle.
 The sine and cosine are returned
 by the Sin and Cos parameters, respectively.
#endif*)
{procedure sincos(val: double; var sin_val, cos_val: double);
begin
  SinCos(val, sin_val, cos_val);
end;}


    function ProjectiontoString(todoProjection :Integer): string;
    var  str :string;
    Begin
    // TODO: Make this better
    str := 'DATUM: ';
    str :=str+ datumToString(1{ d_datum} );
    str :=str+ '\r\nUNIT: ';
    str :=str+ unitToString( 2{d_unit} );
    str :=str+ '\r\n';
    Result:=str;
    End;

    function datumToString( todoDATUM :Integer ):string;
    Begin
    Case  todoDATUM  of
         0: {ADINDAN:}
            Result:= 'ADINDAN';
         1: {ARC1950:}
            Result:= 'ARC1950';
         2: {ARC1960:}
            Result:= 'ARC1960';
         3: {AUSTRALIAN_GEODETIC_1966:}
            Result:= 'AUSTRALIAN GEODETIC 1966';
         4: {AUSTRALIAN_GEODETIC_1984:}
            Result:= 'AUSTRALIAN GEODETIC 1984';
         5: {CAMP_AREA_ASTRO:}
            Result:= 'CAMP AREA ASTRO';
         6: {CAPE:}
            Result:= 'CAPE';
         7: {EUROPEAN_DATUM_1950:}
            Result:= 'EUROPEAN DATUM 1950';
         8: {EUROPEAN_DATUM_1979:}
            Result:= 'EUROPEAN DATUM 1979';
         9: {GEODETIC_DATUM_1949:}
            Result:= 'GEODETIC DATUM 1949';
         10: {HONG_KONG_1963:}
            Result:= 'HONG KONG 1963';
         11: {HU_TZU_SHAN:}
            Result:= 'HU TZU SHAN';
         12: {INDIAN:}
            Result:= 'INDIAN';
         13: {NAD27:}
            Result:= 'NAD27';
         14: {NAD83:}
            Result:= 'NAD83';
         15: {OLD_HAWAIIAN_MEAN:}
            Result:= 'OLD HAWAIIAN MEAN';
         16: {OMAN:}
            Result:= 'OMAN';
         17: {ORDNANCE_SURVEY_1936:}
            Result:= 'ORDNANCE SURVEY 1936';
         18: {PULKOVO_1942:}
            Result:= 'PULKOVO 1942';
         19: {PROVISIONAL_S_AMERICAN_1956:}
            Result:= 'PROVISIONAL SOUTH AMERICAN 1956';
         20: {TOKYO:}
            Result:= 'TOKYO';
         21: {WGS_72:}
            Result:= 'WGS72';
         22: {WGS_84:}
            Result:= 'WGS84';
         23: {NO_DATUM:}
            Result:= 'NO DATUM';
        else
            Result:= 'Unknown Datum';
       end;
    End;

    function unitToString( todoUNIT  :Integer ):string;
    Begin
    case todoUNIT of
         0{RADIANS}:
            Result:= 'radians';
        1 {US_FEET}:
            Result:= 'feet';
        2 {METERS}:
            Result:= 'meters';
        3 {ARC_SECONDS}:
            Result:= 'arc seconds';
        4 {ARC_DEGREES}:
            Result:= 'arc degrees';
        5 {INTERNATIONAL_FEET}:
            Result:= 'international feet';
        6 {STATE_ZONE_TABLE}:
            Result:= 'state zone table units';
        else{default}
            Result:= 'unknown units';
    end;{Case}
    End;

    function packedDMSToDouble(  packedDMS:double ):double;
    var  min,deg,sign:Double;
    Begin
    sign := 1.0;
    if ( packedDMS < 0.0 )then
    begin
        sign := -1.0;
        packedDMS := -packedDMS;
    end;
    deg := packedDMS / 1000000;
    packedDMS := packedDMS-(deg * 1000000);
    min := packedDMS / 1000;
    packedDMS := packedDMS-(min * 1000);
    result := deg;
    result := result+(min / 60.0);
    result := result+(packedDMS / 3600.0);
    result := result*sign;
    End;

(*
    function packedDMSToString(  packedDMS:double;  bLat:boolean ):string;
    Begin
{string
Projection::packedDMSToString( double packedDMS, bool bLat )
begin
    char nsFlag = 'N';
    char ewFlag = 'E';
    if ( packedDMS < 0 )
begin
        nsFlag = 'S';
        ewFlag = 'W';
        packedDMS *= -1;
end

    short deg = packedDMS / 1000000;
    packedDMS -= deg * 1000000;
    short min = packedDMS / 1000;
    packedDMS -= min * 1000;
    double sec = packedDMS;

    char flag = bLat ? nsFlag : ewFlag;
    char strTemp[20];
    sprintf( strTemp, "%d %02d\' %05.2f\" %c", deg, min, sec, flag );

    return strTemp;
}
    Result:='';
    End;
*)






end.
